@@ -204,14 +204,17 @@ namespace libtorrent {
 			: torrent_info(span<char const>{buffer, size}, ec, from_span) {}
 		TORRENT_DEPRECATED
 		explicit torrent_info(lazy_entry const& torrent_file);
+
 		TORRENT_DEPRECATED
 		torrent_info(lazy_entry const& torrent_file, error_code& eca);
 		// all wstring APIs are deprecated since 0.16.11 instead, use the wchar
 		// -> utf8 conversion functions and pass in utf8 strings
+#ifdef TORRENT_WINDOWS
 		TORRENT_DEPRECATED
 		torrent_info(std::wstring const& filename, error_code& ec);
 		TORRENT_DEPRECATED
 		explicit torrent_info(std::wstring const& filename);
+#endif
 #endif // TORRENT_ABI_VERSION
 
 		// frees all storage associated with this torrent_info object
@@ -316,8 +319,7 @@ namespace libtorrent {
 		// or http seed.
 		//
 		// ``add_url_seed()`` and ``add_http_seed()`` adds one url to the list of
-		// url/http seeds. Currently, the only transport protocol supported for
-		// the url is http.
+		// url/http seeds.
 		//
 		// ``set_web_seeds()`` replaces all web seeds with the ones specified in
 		// the ``seeds`` vector.
@@ -579,6 +581,8 @@ namespace libtorrent {
 
 		// ``metadata()`` returns a the raw info section of the torrent file. The size
 		// of the metadata is returned by ``metadata_size()``.
+		// Even though the bytes returned by ``metadata()`` are not ``const``,
+		// they must not be modified.
 		int metadata_size() const { return m_info_section_size; }
 		boost::shared_array<char> metadata() const
 		{ return m_info_section; }
